import numpy as np
import numpy.linalg as la
import matplotlib.pyplot as plt
import pandas as pd
from os import path

GENRES = ["romance", "science fiction", "crime", "western"]
DATA_FILE_PATH = path.abspath(path.join(path.dirname(__file__), "../../data/wiki_movie_plots_deduped_sample.csv"))

_df, _df_train, _df_real_world = None, None, None

def load_df():
    global _df
    if _df is None:
        _df = pd.read_csv(DATA_FILE_PATH)
    return _df

def load_df_train():
    global _df_train, _df
    if _df_train is None:
        load_df()
        _df_train = _df[_df["split"] == "train"] # type: ignore
        _df_train = _df_train.reset_index(drop=True)

    return _df_train

def load_df_real_world():
    global _df_real_world, _df
    if _df_real_world is None:
        load_df()
        _df_real_world = _df[_df["split"] == "test"] # type: ignore
        _df_real_world.reset_index(drop=True)

    return _df_real_world


def dist_coseno(x_idx: int, y_idx: int) -> float:
    return 1 - distances[x_idx, y_idx]

def distCoseno(a: np.ndarray, b: np.ndarray) -> float:
    a_mean = np.sum(a) / len(a)
    b_mean = np.sum(b) / len(b)
    coc = np.dot(a - a_mean, a - a_mean) * np.dot(b - b_mean, b - b_mean)
    if coc == 0:
        # print("div por 0")
        # A PREGUNTAR
        return 0
    corr = np.dot(a - a_mean, b - b_mean) / np.sqrt(coc)
    return 1 - corr

def distCoseno2(a: np.ndarray, b: np.ndarray) -> float:
    return 1 - np.dot(a, b) / (la.norm(a) * la.norm(b))

def dataFrameToVectors(dataframe, Q=1000):
    tokens = np.hstack(dataframe["tokens"].apply(lambda x: x.split()).values)
    unique_tokens = pd.Series(tokens).value_counts().index[:Q].values
    unique_tokens_dict = dict(zip(unique_tokens, range(len(unique_tokens))))
    result = np.zeros((len(dataframe), len(unique_tokens)), dtype=np.float64) #TODO: USE int?

    for i, row in dataframe.iterrows():
        for token in row["tokens"].split():
            if unique_tokens_dict.get(token, False) != False:
               result[i, unique_tokens_dict[token]] += 1
    return result

df_train = load_df_train()

T = dataFrameToVectors(df_train, 1000)
M = np.mean(T, axis=0)
X = T - M.reshape( 1,T.shape[1])
N2 = la.norm(X, axis= 1)

C = (T.T @ T) / (T.shape[1] -1)

Y = (X / N2.reshape(T.shape[0],1))

distances = (Y @ Y.T)

if __name__ == "__main__":
    for i in range(0, 320):
        print(dist_coseno(i,i) - distCoseno(T[i],T[i]))
        # print("distancia numerico", Y[0,i] - Y_[0,i])
