import numpy as np
import numpy.linalg as la
import matplotlib.pyplot as plt
import pandas as pd
from os import path

from classifier.auxiliars import load_df_train

class BagWordModels():
    def __init__(self, df_train, Q = 1000) -> None:
        self.Q = Q

        self.T = self.dataFrameToVectors(df_train)
        self.M = np.mean(self.T, axis=0)
        self.X = self.T - self.M.reshape( 1,self.T.shape[1])
        self.N2 = la.norm(self.X, axis= 1)

        self.C = (self.T.T @ self.T) / (self.T.shape[1] - 1)

        self.Y = (self.X / self.N2.reshape(self.T.shape[0],1))

        self.distances = self.Y @ self.Y.T

    def dist_coseno(self, x_idx: int, y_idx: int) -> float:
        return 1 - self.distances[x_idx, y_idx]

    def dataFrameToVectors(self, dataframe):
        tokens = np.hstack(dataframe["tokens"].apply(lambda x: x.split()).values)
        unique_tokens = pd.Series(tokens).value_counts().index[:self.Q].values
        unique_tokens_dict = dict(zip(unique_tokens, range(len(unique_tokens))))
        result = np.zeros((len(dataframe), len(unique_tokens)), dtype=np.float64) #TODO: USE int?

        for i, row in dataframe.iterrows():
            for token in row["tokens"].split():
                if unique_tokens_dict.get(token, False) != False:
                    result[i, unique_tokens_dict[token]] += 1
        return result
