import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from classifier.knn import knn
from classifier.auxiliars import GENRES

def cross_validation(x_train, k=5, Q=1000):
    df_by_genre = {g: x_train[x_train["Genre"] == g] for g in GENRES}

    class_size = len(df_by_genre["western"]) // 5
    classes = [
        pd.concat([df_by_genre[g].iloc[class_size * i : class_size * (i + 1)] for g in GENRES])
        for i in range(5)
    ]

    xtrains = classes.copy()
    res = np.zeros(5, dtype=np.float64)
    for fold in range(5):
        xtest = xtrains.pop(0)
        res[fold] = knn(xtest, pd.concat(xtrains), k, Q)
        xtrains.append(xtest)

    return np.mean(res)