import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from classifier.auxiliars import *
from classifier.auxiliars import GENRES
from classifier.bag_of_words import BagWordModels

df_train = load_df_train()

# Dado dataset de testeo de n elementos y un dataset de entrenamiento, un k y un Q
# devuelve un array de n elementos su genero inferido mediante knn
def calcularDistancias(xtest, xtrains, k, datamodel):
    distArr = [[None] * len(xtrains)] * len(xtest)
    for i, r_test in enumerate(xtest.iloc()):
        for j, r_train in enumerate(xtrains.iloc()):
            dist = datamodel.dist_coseno(r_test.name, r_train.name)
            distArr[i][j] = (dist, r_train["Genre"]) # type: ignore

        distArr[i].sort(key=lambda x: x[0]) # type: ignore
        distArr[i] = distArr[i][0:k]
        #[(0.53,"western"),...] nxk (distancia,posicion en el dataset de train)
        #[(0.52,157),...]
    distArr = max_genero(distArr)
        #["western",...] nx1
    return distArr

# recibe una matriz de n x k y devuelve un array de n elementos con maximo de los generos k mas cercanos
def max_genero(dist):
    gen = [None] * len(dist)
    for i, entries in enumerate(dist):
        per_genre = [ (g, sum(entry_genre == g for _, entry_genre in entries)) for g in GENRES ]
        gen[i] = max(per_genre, key=lambda a: a[1])[0] # type: ignore
    return gen

def knn(x_test, x_train, k: int, datamodel):
    res = 0
    generosInferidos = calcularDistancias(x_test, x_train, k, datamodel)

    for i in range(len(generosInferidos)):
        if df_train.iloc[x_test.iloc[i].name]["Genre"] == generosInferidos[i]:
            res += 1

    res /= len(generosInferidos)
    return res